@@ -67,7 +67,7 @@
   #:re-export (expand-typerefs
 	       reify-declr reify-decl
 	       udecl->mdecl split-udecl
-	       clean-field-list)
+	       clean-field-list clean-fields)
   #:use-module (nyacc lang c99 cxeval)
   #:use-module (nyacc lang c99 munge-base)
   #:use-module (nyacc lang c99 pprint)
@@ -78,7 +78,7 @@
   #:use-module (sxml match)
   #:use-module (srfi srfi-11)		; let-values
   #:use-module (srfi srfi-2)
-  #:use-module (srfi srfi-1)
+  #:use-module ((srfi srfi-1) #:select (fold fold-right remove))
   #:use-module (system base pmatch)
   ;; debugging:
   #:use-module (system vm trace)
@@ -647,7 +647,7 @@
 	     (loop (cons (sx-list 'enum-defn attr ident `(fixed ,sval)) rez)
 		   (1+ ival) (acons (sx-ref ident 1) sval ddict) (cdr edl)))
 	    (else
-	     (fail "munge/canize-enum-def-list: can't expand ~S"(sx-ref ident 1))
+	     (fail "canize-enum-def-list: can't expand ~S" (sx-ref ident 1))
 	     (loop rez nxt ddict (cdr edl)))))))))))
 
 ;; @deffn {Procecure} enum-ref enum-def-list name => string
